// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef IREE_HAL_LLVMJIT_EXECUTABLE_CACHE_H_
#define IREE_HAL_LLVMJIT_EXECUTABLE_CACHE_H_

#include "iree/hal/allocator.h"
#include "iree/hal/executable.h"
#include "iree/hal/executable_cache.h"
#include "llvm/ExecutionEngine/Orc/LLJIT.h"

namespace iree {
namespace hal {
namespace llvmjit {

class LLVMJITExecutableCache final : public ExecutableCache {
 public:
  explicit LLVMJITExecutableCache(hal::Allocator* allocator,
                                  llvm::orc::LLJIT* execution_engine);
  ~LLVMJITExecutableCache() override;

  bool CanPrepareFormat(ExecutableFormat format) const override;

  StatusOr<ref_ptr<Executable>> PrepareExecutable(
      ExecutableCachingModeBitfield mode, const ExecutableSpec& spec) override;

 private:
  hal::Allocator* allocator_;
  llvm::orc::LLJIT* execution_engine_;
};
}  // namespace llvmjit
}  // namespace hal
}  // namespace iree

#endif  // IREE_HAL_LLVMJIT_EXECUTABLE_CACHE_H_
